
package org.mule.modules.ion.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.ion.config.DeployApplicationMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class DeployApplicationDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public DeployApplicationDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(DeployApplicationMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("file-ref")!= null)&&(!StringUtils.isBlank(element.getAttribute("file-ref")))) {
            if (element.getAttribute("file-ref").startsWith("#")) {
                builder.addPropertyValue("file", element.getAttribute("file-ref"));
            } else {
                builder.addPropertyValue("file", (("#[registry:"+ element.getAttribute("file-ref"))+"]"));
            }
        }
        if ((element.getAttribute("domain")!= null)&&(!StringUtils.isBlank(element.getAttribute("domain")))) {
            builder.addPropertyValue("domain", element.getAttribute("domain"));
        }
        if ((element.getAttribute("muleVersion")!= null)&&(!StringUtils.isBlank(element.getAttribute("muleVersion")))) {
            builder.addPropertyValue("muleVersion", element.getAttribute("muleVersion"));
        }
        if ((element.getAttribute("workerCount")!= null)&&(!StringUtils.isBlank(element.getAttribute("workerCount")))) {
            builder.addPropertyValue("workerCount", element.getAttribute("workerCount"));
        }
        Element environmentVariablesListElement = null;
        environmentVariablesListElement = DomUtils.getChildElementByTagName(element, "environment-variables");
        List<Element> environmentVariablesListChilds = null;
        if (environmentVariablesListElement!= null) {
            String environmentVariablesRef = environmentVariablesListElement.getAttribute("ref");
            if ((environmentVariablesRef!= null)&&(!StringUtils.isBlank(environmentVariablesRef))) {
                if ((!environmentVariablesRef.startsWith(patternInfo.getPrefix()))&&(!environmentVariablesRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("environmentVariables", new RuntimeBeanReference(environmentVariablesRef));
                } else {
                    builder.addPropertyValue("environmentVariables", environmentVariablesRef);
                }
            } else {
                ManagedMap environmentVariables = new ManagedMap();
                environmentVariablesListChilds = DomUtils.getChildElementsByTagName(environmentVariablesListElement, "environment-variable");
                if (environmentVariablesListChilds!= null) {
                    if (environmentVariablesListChilds.size() == 0) {
                        environmentVariablesListChilds = DomUtils.getChildElements(environmentVariablesListElement);
                    }
                    for (Element environmentVariablesChild: environmentVariablesListChilds) {
                        String environmentVariablesValueRef = environmentVariablesChild.getAttribute("value-ref");
                        String environmentVariablesKeyRef = environmentVariablesChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((environmentVariablesValueRef!= null)&&(!StringUtils.isBlank(environmentVariablesValueRef))) {
                            valueObject = new RuntimeBeanReference(environmentVariablesValueRef);
                        } else {
                            valueObject = environmentVariablesChild.getTextContent();
                        }
                        if ((environmentVariablesKeyRef!= null)&&(!StringUtils.isBlank(environmentVariablesKeyRef))) {
                            keyObject = new RuntimeBeanReference(environmentVariablesKeyRef);
                        } else {
                            keyObject = environmentVariablesChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = environmentVariablesChild.getTagName();
                        }
                        environmentVariables.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("environmentVariables", environmentVariables);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
